diff -r 1c4af66bc62f src/lib-sieve/sieve-extensions.c
--- a/src/lib-sieve/sieve-extensions.c	Wed May 20 12:20:33 2009 +0200
+++ b/src/lib-sieve/sieve-extensions.c	Tue May 26 09:21:56 2009 +0200
@@ -357,11 +357,14 @@
 void sieve_extensions_set_string(const char *ext_string)
 {
 	ARRAY_DEFINE(enabled_extensions, const struct sieve_extension_registration *);
+	ARRAY_DEFINE(disabled_extensions, const struct sieve_extension_registration *);
 	const struct sieve_extension_registration *const *ena_eregs;
+	const struct sieve_extension_registration *const *dis_eregs;
 	struct sieve_extension_registration *eregs;
 	const char *bp = ext_string;
 	const char *ep = bp;
-	unsigned int i, ext_count, ena_count;
+	unsigned int i, ext_count, ena_count, dis_count;
+	bool relative_changes = FALSE;
 
 	if ( ext_string == NULL ) {
 		/* Enable all */
@@ -374,6 +377,7 @@
 	}
 
 	t_array_init(&enabled_extensions, array_count(&extensions));
+	t_array_init(&disabled_extensions, array_count(&extensions));
 
 	do {
 		const char *name;
@@ -388,7 +392,14 @@
 
 		if ( *name != '\0' ) {
 			const struct sieve_extension_registration *ereg;
+			char op = '\0';	/* no add / remove operation */
 	
+			if ( *name == '+' 		/* add to existing config */
+			 || *name == '-' ) {	/* remove from existing config */
+			 	op = *name++;
+			 	relative_changes = TRUE;
+			}
+
 			if ( *name == '@' )
 				ereg = NULL;
 			else
@@ -401,7 +412,10 @@
 				continue;
 			}
 
-			array_append(&enabled_extensions, &ereg, 1);
+			if(op == '-')
+				array_append(&disabled_extensions, &ereg, 1);
+			else
+				array_append(&enabled_extensions, &ereg, 1);
 		}
 
 	} while ( *bp == '\0' || ep != NULL );
@@ -409,26 +423,38 @@
 	eregs = array_get_modifiable(&extensions, &ext_count);
 	ena_eregs = array_get(&enabled_extensions, &ena_count);
 
-	/* Set new extension status */
-	for ( i = 0; i < ext_count; i++ ) {
-		unsigned int j;
-		bool disabled = TRUE;
-
-		for ( j = 0; j < ena_count; j++ ) {
-			if ( ena_eregs[j] == &eregs[i] ) {
-				disabled = FALSE;
-				break;
-			}		
+	if(relative_changes) {	/* add / remove extensions */
+		dis_eregs = array_get(&disabled_extensions, &dis_count);
+		for(i = 0; i < dis_count; ++i) {
+			if(!dis_eregs[i]->required) {
+				sieve_extension_disable(dis_eregs[i]);
+			}
 		}
-
-		if ( eregs[i].extension->_id != NULL && 
-			*(eregs[i].extension->name) != '@' ) {
-			if ( disabled && !eregs[i].required )
-				sieve_extension_disable(&eregs[i]);
-			else
-				sieve_extension_enable(&eregs[i]);
+		for(i = 0; i < ena_count; ++i) {
+			sieve_extension_enable(ena_eregs[i]);
 		}
 	}
+	else
+		/* Set new extension status */
+		for ( i = 0; i < ext_count; i++ ) {
+			unsigned int j;
+			bool disabled = TRUE;
+
+			for ( j = 0; j < ena_count; j++ ) {
+				if ( ena_eregs[j] == &eregs[i] ) {
+					disabled = FALSE;
+					break;
+				}		
+			}
+
+			if ( eregs[i].extension->_id != NULL && 
+				*(eregs[i].extension->name) != '@' ) {
+				if ( disabled && !eregs[i].required )
+					sieve_extension_disable(&eregs[i]);
+				else
+					sieve_extension_enable(&eregs[i]);
+			}
+		}
 }
 
 static void sieve_extensions_deinit_registry(void) 
